//  Demo 24: MarchingSquaresApplet.java
//
//     Another Simple Java Applet to Use the Squares Class
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, June, 1999
//
// Two columns of Squares which march a bit like a marching band.
//
// Illustrates graphics double buffering to avoid animation flicker.
// Also illustrates additional methods of the Square and Squares classes.

import java.awt.*;
import java.applet.Applet;
import Square;  // we are going to do things with individual
                // square objects, so, by contrast with
                // MySquaresApplet.java, we have to import the
                // Square class
import Squares;

public class MarchingSquaresApplet extends Applet {

   Squares s, t;

   Graphics gBuffer;   // Graphics buffer for off-screen drawing
   Image imageBuffer;  // need an Image object to create our
                       // own graphics object, for obscure
                       // Java reasons that only Sun understands

   public void init() {

     // two columns of 10 20 by 20 squares spaced 60
     // pixels apart
     s = new Squares(50, 50, 20, Color.red, 60, 5);
     t = new Squares(250, 80, 20, Color.blue, 60, 5);

     // create the gBuffer Graphics buffer object
     //   two step process, more Java obscurity
     imageBuffer = createImage(400, 400);  // 400 by 400 pixels
     gBuffer = imageBuffer.getGraphics();
   }

   public void paint(Graphics g) {

      for(int i = 1; i <= 25; i++) {

        marchAcross(g, 8);
        pulsate(g, 4);
      }

   } // paint


   // march across screen n pixels to right and left
   public void marchAcross(Graphics g, int n) {

     Color backgroundColor = getBackground();

     for(int i = 1; i <= n; i++) {

        // move -- do drawing offscreen on the Graphics
        //         buffer object
        s.draw(gBuffer, backgroundColor);  // hide squares
        t.draw(gBuffer, backgroundColor);  // hide squares
        s.move(1, 0);
        t.move(-1, 0);
        s.draw(gBuffer);
        t.draw(gBuffer);

        //draw completed image on the screen
        g.drawImage(imageBuffer, 0, 0, this);

        try {
          Thread.sleep(50);  // 50 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }

     } // for
   } // marchAcross

   // pulsate n times
   public void pulsate(Graphics g, int n) {

     Color backgroundColor = getBackground();
 
     for(int i = 1; i <= n; i++) {

        s.draw(gBuffer, backgroundColor);  // hide squares
        t.draw(gBuffer, backgroundColor);  // hide squares 
        s.expand(-5);
        t.expand(-5);
        s.draw(gBuffer);
        t.draw(gBuffer);

        //draw completed image on the screen
        g.drawImage(imageBuffer, 0, 0, this);

        try {
          Thread.sleep(500);  // 500 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }

        s.draw(g, backgroundColor);  // hide squares again
        t.draw(g, backgroundColor);  // hide squares again
        s.expand(5);
        t.expand(5);
        s.draw(g);
        t.draw(g);

        try {
          Thread.sleep(500);  // 500 ms pause
        }
        catch (InterruptedException e) {
          // do nothing
        }

     } // for 
   } // pulsate()
} // class MySquares
        